<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class AssistenteIntegracaoCardapio extends \PME\Services\TeknisaCrud {
    protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $connection;
	protected $newCode;
	protected $environment;
	protected $entityManager;
	protected $elaboracaoCardapio;
	protected $utilitiesCard;

    public function __construct($crudService, $parameterBag, $connection, $newCode, $environment, $entityManager, $elaboracaoCardapio, $utilitiesCard) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->elaboracaoCardapio = $elaboracaoCardapio;
        $this->utilitiesCard = $utilitiesCard;
	}
	
	public function setDataSourceGridDestino(DTO\Request $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
	    $centralDest = $parameters['formCentralDest'];
	    $dadosOrigem = $parameters['formDadosOrigem'];
	    
	    $cdServFili = !empty($dadosOrigem['CDSERVFILI']) ? implode('_', $dadosOrigem['CDSERVFILI']) : 'T';
	    $cdGrupTipoCard = isset($dadosOrigem['CDGRUPTIPOCARD']) ? $dadosOrigem['CDGRUPTIPOCARD'] : 'T';
	    $cdTipoCard = !empty($dadosOrigem['TPCARD']) ? implode('_', $dadosOrigem['TPCARD']) : 'T';
	
	    $params = array(
	        "P_DATAFIN"           => $dadosOrigem['DTATE'],
	        "P_CDFILCENTPME"      => $dadosOrigem['CENTRAL_ORIGEM'],
	        "P_CDSERVFILI"        => $cdServFili,
	        "P_DATAINI"           => $dadosOrigem['DTDE'],
	        "P_CDAGRUPTPCARD"     => $cdGrupTipoCard,
	        "P_CDTIPOCARD"        => $cdTipoCard,
	        "P_CDFILCENTPME_DEST" => $centralDest['CENTRAL_DESTINO']
	    );
	
	    $query = "SELECT DISTINCT 
	                CD.NRCARDPME || CD.CDSERVFILI AS PK, 
	                C.NRCARDPME, 
	                T.CDTIPOCARD, 
	                T.NMTIPOCARD, 
	                T.CDTIPOCARD||' | '||T.NMTIPOCARD AS TIPOCARD, 
	                S.CDSERVFILI, 
	                S.NMSERVICO, 
	                S.CDSERVFILI||' | '||S.NMSERVICO AS SERVICO, 
	                TO_CHAR(DTINICARDPME, 'DD/MM/YYYY') AS DTINICARDPME, 
	                TO_CHAR(DTFINCARDPME, 'DD/MM/YYYY') AS DTFINCARDPME, 
	                TO_CHAR(DTINICARDPME, 'DD/MM/YYYY') AS DTINIPERI, 
	                --TO_CHAR(DTINICARDPME, 'Dy') AS DIASEMANA, 
	                '' AS DIASEMANA,
	                C.DSCARDPME, 
	                DECODE(:P_CDFILCENTPME, :P_CDFILCENTPME_DEST, T.CDTIPOCARD, NULL) AS TPCARD_DESTINO, 
	                DECODE(:P_CDFILCENTPME, :P_CDFILCENTPME_DEST, T.NMTIPOCARD, NULL) AS NMTIPOCARDDEST, 
	                DECODE(:P_CDFILCENTPME, :P_CDFILCENTPME_DEST, S.CDSERVFILI, NULL) AS CDSERVFILI_DESTINO, 
	                DECODE(:P_CDFILCENTPME, :P_CDFILCENTPME_DEST, S.NMSERVICO, NULL) AS NMSERVICODEST, 
	                '' AS DTDE_DESTINO, 
	                '' AS DTATE_DESTINO, 
	                '' AS DTPERINI_DESTINO, 
	                '' AS DIASEMANA_DESTINO, 
	                '' AS DSCARDPME_DESTINO, 
	                :P_CDFILCENTPME_DEST AS CENTRAL_DESTINO, 
	                C.NRORG 
	              FROM 
	                CARDIAPME CD 
	              INNER JOIN CARDAPIOPME C 
	                ON C.CDFILCENTPME = CD.CDFILCENTPME AND C.NRCARDPME = CD.NRCARDPME 
	              INNER JOIN SERVICO S 
	                ON S.CDFILIAL = CD.CDFILIAL AND S.CDSERVFILI = CD.CDSERVFILI 
	              INNER JOIN TIPOCARD T 
	                ON T.CDFILCENTPME = C.CDFILCENTPME AND T.CDTIPOCARD = C.CDTIPOCARD AND T.IDTPCARDATIV = 'S' 
	              WHERE 
	                (CD.CDFILCENTPME = CD.CDFILIAL) 
	                AND (CD.CDFILCENTPME = :P_CDFILCENTPME) 
	                AND (C.DTINICARDPME >= TO_DATE(:P_DATAINI, 'DD/MM/YYYY')) 
	                AND (C.DTFINCARDPME <= TO_DATE(:P_DATAFIN, 'DD/MM/YYYY')) 
	                AND (T.CDGRUPTIPOCARD = :P_CDAGRUPTPCARD OR :P_CDAGRUPTPCARD = 'T') 
	                AND ((:P_CDSERVFILI = 'T') OR INSTR(:P_CDSERVFILI, CD.CDSERVFILI) > 0) 
	                AND ((:P_CDTIPOCARD = 'T') OR INSTR(:P_CDTIPOCARD, C.CDTIPOCARD) > 0) 
	              ORDER BY 
	                DTINICARDPME, T.CDTIPOCARD, S.CDSERVFILI";
	
	    $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}

	
	public function insertListCardAssisIntegra(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
	    $rows = $parameters['rows'];
	    $this->connection->beginTransaction();
	
	    try {
	        foreach ($rows as $row) {
	            $nrorg = $row['NRORG'];
	            $nrlista = $this->newCode->getCodeByTableName('LISTA_DTCARDIAPME', $nrorg);
	            
	            $query = "INSERT INTO LISTA_DTCARDIAPME (NRLISTA, DTCARDIAPME, DTCARDIADESTPME) VALUES (:P_NRLISTA, :P_DTCARDIAPME, :P_DTCARDIADESTPME)";
	            $params = array(
	                "P_NRLISTA"           => $nrlista,
	                "P_DTCARDIAPME"       => $row['DTINICARDPME'],
	                "P_DTCARDIADESTPME"   => $row['DTDE_DESTINO']
	            );
	            
	            $stmt = $this->connection->prepare($query);
	            $stmt->execute($params);
	
	            $this->callExportaCardPae($row, $nrlista, $response);
	        }
	        $this->connection->commit();
	    } catch (\Exception $e) {
	        $this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
	    }
	}
		
	public function callExportaCardPae($row, $nrlista, DTO\Response $response){
		$connection = $this->entityManager->getConnection();
		$procedure = new StoredProcedure($connection, 'EXPORTA_CARDAPIO_PAE');

		$procedure->addParam(new Param('P_CDCENTRAL',        Param::PARAM_INPUT, $row['CENTRAL_ORIGEM'],     Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRCARDPME',        Param::PARAM_INPUT, $row['NRCARDPME'],          Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDTIPOCARD',       Param::PARAM_INPUT, $row['CDTIPOCARD'],         Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NMTIPOCARD',       Param::PARAM_INPUT, $row['NMTIPOCARD'],         Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDSERVFILI',       Param::PARAM_INPUT, $row['CDSERVFILI'],         Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NMSERVICO',        Param::PARAM_INPUT, $row['NMSERVICO'],          Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTINICARDPME',     Param::PARAM_INPUT, $row['DTINICARDPME'],       Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTFINCARDPME',     Param::PARAM_INPUT, $row['DTFINCARDPME'],       Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DSCARDPME',        Param::PARAM_INPUT, $row['DSCARDPME'],          Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDTIPOCARDDEST',   Param::PARAM_INPUT, $row['TPCARD_DESTINO'],     Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NMTIPOCARDDEST',   Param::PARAM_INPUT, $row['NMTIPOCARDDEST'],     Param::PARAM_TYPE_STR)); 
		$procedure->addParam(new Param('P_CDSERVFILIDEST',   Param::PARAM_INPUT, $row['CDSERVFILI_DESTINO'], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NMSERVICODEST',    Param::PARAM_INPUT, $row['NMSERVICODEST'],      Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTINICARDPMEDEST', Param::PARAM_INPUT, $row['DTDE_DESTINO'],       Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTFINCARDPMEDEST', Param::PARAM_INPUT, $row['DTATE_DESTINO'],      Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DSCARDPMEDEST',    Param::PARAM_INPUT, $row['DSCARDPME_DESTINO'],  Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRLISTACARDIAPME', Param::PARAM_INPUT, $nrlista,                   Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRLOGINTCARD',     Param::PARAM_INPUT, NULL,                       Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDCENTRALDEST',    Param::PARAM_INPUT, $row['CENTRAL_DESTINO'],    Param::PARAM_TYPE_STR));
		$procedure->execute();
	}
	
	/* public function validaCentralIntegracaoCard(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    $query = "SELECT A.CDFILIAL, A.NMFILIAL, A.CDEMPRESA, A.CDINSCESTA, A.CDINSCMUNI, B.DTLIMPEZA, B.DTPROCESSA, B.DTPROCESSFIN FROM FILIAL A, PARAMFILIAL B WHERE A.CDFILIAL = :CDFILIAL AND A.CDFILIAL = B.CDFILIAL (+)";
	    $params = array(
            "CDFILIAL"   => $row['CDFILCENTPME']
        );
        
        $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
		
	}*/
	
	public function dataProcessamentoCentral(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	     $query = "SELECT DECODE(PG.IDUTILDTPROC, 'S', NVL(TO_CHAR(PF.DTPROCESSA, 'DD/MM/YYYY'), TO_CHAR(SYSDATE, 'DD/MM/YYYY')), TO_CHAR(SYSDATE, 'DD/MM/YYYY')) AS DTPROCESSA 
              FROM PARAMFILIAL PF, PARAMGERAL PG 
              WHERE PF.CDFILIAL = :P_CDFILIAL 
              AND ROWNUM = 1";
	    $params = array(
            "P_CDFILIAL"   => $row['CDFILCENTPME']
        );
        
        $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		$dataProcessamento = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $dataProcessamento));
		
	}
	
	public function verificaCardapioPeriodo(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
        $rows = $parameters['rows'];
	    $query = "SELECT CDFILCENTPME, TO_CHAR(DTINICARDPME, 'DD/MM/YYYY') AS DTINICARDPME, TO_CHAR(DTFINCARDPME, 'DD/MM/YYYY') AS DTFINCARDPME, CDTIPOCARD FROM CARDAPIOPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDTIPOCARD = :P_CDTIPOCARD AND (:P_DTINICARD BETWEEN DTINICARDPME AND DTFINCARDPME OR :P_DTFINCARD BETWEEN DTINICARDPME AND DTFINCARDPME OR DTINICARDPME BETWEEN :P_DTINICARD AND :P_DTFINCARD OR DTFINCARDPME BETWEEN :P_DTINICARD AND :P_DTFINCARD)";
	    
	    foreach($rows as $row) {
		    $params = array(
	            "P_DTFINCARD"    => $row['DTATE_DESTINO'],
	            "P_CDFILCENTPME" => $row['CENTRAL_DESTINO'],
	            "P_CDTIPOCARD"   => $row['TPCARD_DESTINO'],
	            "P_DTINICARD"    => $row['DTDE_DESTINO']
	        );
	    }
        
        $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}
	
	public function verificaConfigCardServ(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
        $rows = $parameters['rows'];
        
	    $query = "SELECT CDFILCENTPME, CDTIPOCARD, CDSERVFILI, NRSEQCNFTCAR, CDTIPOPRAT, IDOPCCNFTCAR, IDIMPCNFTCAR, NRORDCNFTCAR FROM CONFPADTCARD WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDTIPOCARD = :P_CDTIPOCARD AND CDSERVFILI = :P_CDSERVFILI ORDER BY NRORDCNFTCAR";
	    
	    foreach($rows as $row) {
		    $params = array(
	            "P_CDFILCENTPME" => $row['CENTRAL_DESTINO'],
	            "P_CDTIPOCARD" => $row['TPCARD_DESTINO'],
	            "P_CDSERVFILI" => $row['CDSERVFILI_DESTINO']
	        );
	    }
        
        $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
		
	}
	
	public function validaCustoPadrao(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
	    $rows = $parameters['rows'];
	    $query = "SELECT CDFILCENTPME, CDTIPOCARD, CDSERVFILI, TO_CHAR(DTREFCUSTCAR, 'DD/MM/YYYY') AS DTREFCUSTCAR, VRCUSPADTCAR, NRORG 
	              FROM CUSTPADTCARD 
	              WHERE CDFILCENTPME = :CDFILCENTPME AND CDTIPOCARD = :CDTIPOCARD AND CDSERVFILI = :CDSERVFILI
	              AND DTREFCUSTCAR <= :DTREFCUSTCAR
	              ORDER BY DTREFCUSTCAR DESC";
	
	    $finalResult = [];
	
	    foreach ($rows as $row) {
	        $params = array(
	            "CDFILCENTPME" => $row['CENTRAL_DESTINO'],
	            "CDSERVFILI"   => $row['CDSERVFILI_DESTINO'],
	            "CDTIPOCARD"   => $row['TPCARD_DESTINO'],
	            "DTREFCUSTCAR" => $row['DTDE_DESTINO']
	        );
	
	        $stmt = $this->connection->prepare($query);
	        $stmt->execute($params);
	        $result = $stmt->fetchAll();
	
	        if (!empty($result)) {
	            $finalResult = array_merge($finalResult, $result);
	        }
	    }
	    $response->addDataSet(new DataSet("result", $finalResult));
	}

}