<?php
namespace PME\Controllers\Parametrizacao;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class RealocacaoComensaisEspeciaisPrato extends \PME\Services\TeknisaCrud {
    protected $crudService;
	protected $parameterBag;
	protected $connection;
	protected $environment;
	protected $newCode;
	
	public function __construct($crudService, $parameterBag, $connection, $environment, $newCode) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->environment = $environment;
        $this->newCode     = $newCode;
	}
	
	public function saveRealComensaisEspeciaisPrato(DTO\Request $request, DTO\Response $response) {
	    $this->connection->beginTransaction();
	    try {
	        $params = $request->getParameters();
	        $nrorg = $this->environment->getNrOrg();
	        
	       // $queryUpdate = "UPDATE PERACEITPRATO SET VRPERACEIT = :VRPERACEIT WHERE CDFILCENTPME = :CDFILCENTPME AND CDFILIAL = :CDFILIAL AND CDPRATO = :CDPRATO AND NRORG = :NRORG AND CDTIPOCARD = :CDTIPOCARD";
	        $select = "SELECT CDREALCOMESPPRATPME, RCEP.CDFILCENTPME, RCEP.CDFILIAL, RCEP.CDTIPOCARD, RCEP.CDTIPOPRAT, RCEP.CDPRATO, RCEP.CDFILAUXPRAT, RCEP.NRORG 
	                        FROM REALCOMESPPRATPME RCEP WHERE RCEP.CDFILCENTPME = :CDFILCENTPME AND RCEP.CDFILIAL = :CDFILIAL AND RCEP.CDTIPOCARD = :CDTIPOCARD AND CDTIPOPRAT = :CDTIPOPRAT AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT";
	        $queryInsert = "INSERT INTO REALCOMESPPRATPME (CDREALCOMESPPRATPME, CDFILCENTPME, CDFILIAL, CDTIPOCARD, CDTIPOPRAT, CDPRATO, CDFILAUXPRAT, NRORG) VALUES (:CDREALCOMESPPRATPME, :CDFILCENTPME, :CDFILIAL, :CDTIPOCARD, :CDTIPOPRAT, :CDPRATO, :CDFILAUXPRAT, :NRORG)";
	        
	        foreach ($params['editedRows'] as $data) {
	        	$params = array(
                    "CDFILCENTPME" => $data['CDFILCENTPME'],
                    "CDFILIAL"     => $data['CDFILIAL'],
                    "CDTIPOCARD"   => $data['CDTIPOCARD'],
                    "CDTIPOPRAT"      => $data['CDTIPOPRAT'],
                    "CDPRATO"      => $data['CDPRATO'],
                    "CDFILAUXPRAT"      => $data['CDFILAUXPRAT'],
                    "NRORG"        => $nrorg
                );
                
                if (count($this->connection->fetchAll($select, $params)) == 0) {
                
	    		  $primaryKey = $this->newCode->getCodeByTableName('COMESPPRATPME', $nrorg);
	        	  //$primaryKey = $primaryKey + 1;
	        	  $params['CDREALCOMESPPRATPME'] = $primaryKey;
	        	  $stmt = $this->connection->prepare($queryInsert);
	       
                } else {
                    $message['message'] = 'Impossível adicionar. Registro já existe.';
				    return $response->addDataSet(new DataSet("message", $message));
                }
	        	
	        	$stmt->execute($params);
	        };
	        $this->connection->commit();
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function deleteRealComensaisEspeciaisPrato(DTO\Request $request, DTO\Response $response) {
		$this->connection->beginTransaction();
		try {		
			$params = $request->getParameters()["checkedRows"];
            foreach ($params as $param) {
				$validacao = $this->validaDeleteReal($param);
                
				if ($validacao !== true) {
					$this->connection->rollBack();
					return $response->addDataset(new DataSet('message', array($validacao)));
				}
				
				$sql = "DELETE FROM REALCOMESPPRATPME 
                         WHERE
                         CDFILCENTPME = :CDFILCENTPME
                         AND CDTIPOCARD = :CDTIPOCARD
                         AND CDFILIAL = :CDFILIAL
                         AND CDREALCOMESPPRATPME = :CDREALCOMESPPRATPME
                         AND NRORG = :NRORG";
                         
                        
                        //  AND CDTIPOPRAT = :CDTIPOPRAT 
                        //  AND CDPRATO = :CDPRATO
                        //  AND CDFILAUXPRAT = :CDFILAUXPRAT
                
                $paramsQuery = array(
                    'CDFILCENTPME' => $param['CDFILCENTPME'],
                    'CDFILIAL' => $param['CDFILIAL'],
                    'CDTIPOCARD' => $param['CDTIPOCARD'],
                    // 'CDTIPOPRAT' => $param['CDTIPOPRAT'],
                    // 'CDPRATO' => $param['CDPRATO'],
                    // 'CDFILAUXPRAT' => $param['CDFILAUXPRAT'],
                    'CDREALCOMESPPRATPME' => $param['CDREALCOMESPPRATPME'],
                    'NRORG' => $this->environment->getNrOrg(),
                );
                
                $this->connection->prepare($sql)->execute($paramsQuery);
			}
			
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function validaDeleteReal($param){
		$sql = "SELECT CDCOMESPPRATPME
                  FROM COMESPPRATPME
                 WHERE CDREALCOMESPPRATPME = :CDREALCOMESPPRATPME";
        
        $paramSql = array(
            'CDREALCOMESPPRATPME' => $param['CDREALCOMESPPRATPME']
        );
        
        $resultado = $this->connection->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_TEM_PRATO_REGISTRO';
        }
        
        return true;
	}
	
	public function saveComensaisEspeciaisPrato(DTO\Request $request, DTO\Response $response) {
	    $this->connection->beginTransaction();
	    try {
	        $params = $request->getParameters();
	        $nrorg = $this->environment->getNrOrg();
	        
	        $queryUpdate = "UPDATE COMESPPRATPME SET QTCOMESPRAT = :QTCOMESPRAT WHERE CDCOMESPPRATPME = :CDCOMESPPRATPME AND CDREALCOMESPPRATPME = :CDREALCOMESPPRATPME AND NRORG = :NRORG";
	        $select = "SELECT CDCOMESPPRATPME, CDTIPOPRAT, CDPRATO, CDFILAUXPRAT, QTCOMESPRAT, CDREALCOMESPPRATPME, NRORG, CDREALCOMESPPRATPME
	                        FROM COMESPPRATPME WHERE CDREALCOMESPPRATPME = :CDREALCOMESPPRATPME AND CDTIPOPRAT = :CDTIPOPRAT AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND NRORG = :NRORG";
	        $queryInsert = "INSERT INTO COMESPPRATPME (CDREALCOMESPPRATPME, CDTIPOPRAT, CDPRATO, CDFILAUXPRAT, NRORG, CDCOMESPPRATPME, QTCOMESPRAT) VALUES (:CDREALCOMESPPRATPME, :CDTIPOPRAT, :CDPRATO, :CDFILAUXPRAT, :NRORG, :CDCOMESPPRATPME, :QTCOMESPRAT)";
	        
	        foreach ($params['editedRows'] as $data) {
	        	$params = array(
                    "CDREALCOMESPPRATPME" => $data['CDREALCOMESPPRATPME'],
                    "CDTIPOPRAT"     => $data['CDTIPOPRAT'],
                    "CDPRATO"   => $data['CDPRATO'],
                    "CDFILAUXPRAT"      => $data['CDFILAUXPRAT'],
                    "NRORG"        => $nrorg
                );
                
            	if(in_array('__is_new', $data) && $data['__is_new'] === true) {
                    if (count($this->connection->fetchAll($select, $params)) == 0) {
                      if (!$this->validaRepeticaoPratos($data)) {
                          $message['message'] = 'Não é possível realocar comensais para o próprio prato.';
	                      return $response->addDataSet(new DataSet("message", $message));
                      }
                      $primaryKey = $this->newCode->getCodeByTableName('COMESPPRATPME', $nrorg);
    	        	  $params['CDCOMESPPRATPME'] = $primaryKey;
    	        	  $params['QTCOMESPRAT'] = $data['QTCOMESPRAT'];
    	        	  $stmt = $this->connection->prepare($queryInsert)->execute($params);
                    } else {
                        $message['message'] = 'Impossível adicionar. Registro já existe.';
    				    return $response->addDataSet(new DataSet("message", $message));
                    }
            	}else{
            	    $paramsUpdate = array(
        	            'QTCOMESPRAT' => $data['QTCOMESPRAT'],
        	            'CDCOMESPPRATPME' => $data['CDCOMESPPRATPME'],
        	            'CDREALCOMESPPRATPME' => $data['CDREALCOMESPPRATPME'],
        	            'NRORG' => $nrorg,
        	        );
	        	  $stmt = $this->connection->prepare($queryUpdate);
	        	  $stmt->execute($paramsUpdate);
	        	}
	        	
	        }
	        $this->connection->commit();
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function validaRepeticaoPratos($data) {
	    $nrorg = $this->environment->getNrOrg();
	    $select = "SELECT CDREALCOMESPPRATPME, RCEP.CDFILCENTPME, RCEP.CDFILIAL, RCEP.CDTIPOCARD, RCEP.CDTIPOPRAT, RCEP.CDPRATO, RCEP.CDFILAUXPRAT, RCEP.NRORG 
	                        FROM REALCOMESPPRATPME RCEP WHERE RCEP.CDFILCENTPME = :CDFILCENTPME AND RCEP.CDFILIAL = :CDFILIAL AND RCEP.CDTIPOCARD = :CDTIPOCARD 
                            AND CDREALCOMESPPRATPME = :CDREALCOMESPPRATPME AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT";
	       $params = array(
                    "CDFILCENTPME" => $data['CDFILCENTPME'],
                    "CDFILIAL"     => $data['CDFILIAL'],
                    "CDTIPOCARD"   => $data['CDTIPOCARD'],
                    "CDREALCOMESPPRATPME" => $data['CDREALCOMESPPRATPME'],
                    "CDPRATO" => $data['CDPRATO'],
                    "CDFILAUXPRAT" => $data['CDFILAUXPRAT'],
                    "NRORG"        => $nrorg
                );
        
        if (count($this->connection->fetchAll($select, $params)) > 0) return false;
        return true;
	                       
	}
	
	public function deleteComensaisEspeciaisPrato(DTO\Request $request, DTO\Response $response) {
		$this->connection->beginTransaction();
		try {		
			$params = $request->getParameters()["checkedRows"];
            foreach ($params as $param) {
				$validacao = $this->validaDeleteReal($param);
                
				if ($validacao !== true) {
					$this->connection->rollBack();
					return $response->addDataset(new DataSet('message', array($validacao)));
				}
				
				$sql = "DELETE FROM COMESPPRATPME 
                         WHERE
                         CDCOMESPPRATPME = :CDCOMESPPRATPME
                         AND CDREALCOMESPPRATPME = :CDREALCOMESPPRATPME
                         AND NRORG = :NRORG";
                        
                        //   AND CDTIPOPRAT = :CDTIPOPRAT 
                        //  AND CDPRATO = :CDPRATO
                        //  AND CDFILAUXPRAT = :CDFILAUXPRAT
                        
                $paramsQuery = array(
                    'CDCOMESPPRATPME' => $param['CDCOMESPPRATPME'],
                    // 'CDTIPOPRAT' => $param['CDTIPOPRAT'],
                    // 'CDPRATO' => $param['CDPRATO'],
                    // 'CDFILAUXPRAT' => $param['CDFILAUXPRAT'],
                    'CDREALCOMESPPRATPME' => $param['CDREALCOMESPPRATPME'],
                    'NRORG' => $this->environment->getNrOrg(),
                );
                
                $this->connection->prepare($sql)->execute($paramsQuery);
			}
			
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function callProcedureAjustComEspPrat(DTO\Request\Row $request, DTO\Response $response) {
	    try {
		    $row = $request->getRow();
		    $cdfilcentpme = $row['CDFILCENTPME'];
		    $filial = is_array($row['UNIDADE']) && !empty($row['UNIDADE']) ? implode("_", $row['UNIDADE']) : null;
		    $initialDate = $row['initialDateAjust'];
		    $finalDate = $row['finalDateAjust'];
		    $idPratos = $row['platesRadio'];
		    $acrecTipoPrat = $row['platesCheckBox'] === true ? 'S' : 'N';
		    $tipoCard = $row['CDTIPOCARD'];
		    $nrCardPme = $row['NRCARDPME'];
		    $cdservfili = $row['CDSERVFILI'];
		    
			$procedure = new StoredProcedure($this->connection, 'REALOCAR_COMENSAIS_ESP_PAE'); 
        
            $procedure->addParam(new Param('P_CDCENTRAL', Param::PARAM_INPUT, $cdfilcentpme, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDTIPOCARD', Param::PARAM_INPUT, $tipoCard, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $cdservfili, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCARDPME', Param::PARAM_INPUT, $nrCardPme, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $initialDate, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $finalDate, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDPRATOS', Param::PARAM_INPUT, $idPratos, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ACRESCTIPOPRAT', Param::PARAM_INPUT, $acrecTipoPrat, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLOGINTCARD', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR, 10));
            // $nrlogintcard = $procedure->execute()['P_NRLOGINTCARD'];
            $nrlogintcard = $procedure->execute();
            
            // $result = $this->retornoLogintCard($request, $response, $nrlogintcard);
            // $response->addDataSet(new DataSet("result", $result));
		} catch (\Exception $e) {
	    	$response->setError(new Error($e->getMessage(), 500)); 
		}
	}
	
	public function retornoLogintCard($request, $response, $nrlogintcard) {
	    $sql = "SELECT L.CDFILCENTPME, L.CDTIPOCARD, T.NMTIPOCARD, TO_CHAR(L.DTCARDIAPME, 'DD/MM/YYYY') AS DTCARDIAPME, L.CDSERVFILI, S.NMSERVICO, L.CDFILIAL, L.CDPRATO, L.CDFILAUXPRAT, L.CDPRODUTO, P.NMPRODUTO, L.NRERRO FROM LOGINTCARDPAE L LEFT JOIN TIPOCARD T ON T.CDFILCENTPME = L.CDFILCENTPME AND T.CDTIPOCARD = L.CDTIPOCARD LEFT JOIN SERVICO S ON S.CDFILIAL = L.CDFILCENTPME AND S.CDSERVFILI = L.CDSERVFILI LEFT JOIN PRODUTO P ON P.CDPRODUTO = L.CDPRODUTO WHERE L.NRLOGINTCARDPAE = :P_NRLOGINTCARDPAE";
        $params = array(
          "P_NRLOGINTCARDPAE" => $nrlogintcard
        );
        $stmt = $this->connection->prepare($sql);
    		$stmt->execute($params);
        $result = $stmt->fetchAll();
        
        
        $delete = 'DELETE FROM LOGINTCARDPAE WHERE NRLOGINTCARDPAE = :P_NRLOGINTCARDPAE';
        $params = array(
            "P_NRLOGINTCARDPAE" => $nrlogintcard
        );
        
        $std = $this->entityManager->getConnection()->prepare($delete);
        $std->execute($params);
        
		return $result;
	}
}