<?php
/**
 * Created by PhpStorm.
 * User: paulopimenta
 * Date: 22/01/2015
 * Time: 09:37
 */

namespace Fiscal\Logic\NotaFiscal;

use Contabilidade\Entities\Contopectbl;
use Contabilidade\Entities\Erroctbtemp;
use Contabilidade\Entities\lancctbtemp;
use Contabilidade\Logic\Contabilizacao\Contabilizacao;
use Estoque\Entities\Lanctoestoq;
use Estoque\Logic\AtualizacaoEstoque\AtualizadorDeEstoque;
use Estoque\Logic\AtualizacaoEstoque\CalculaCustoLiquido;
use Financeiro\Entities\Ajctpagar;
use Fiscal\Entities\Imposnftot;
use Fiscal\Entities\Impostonf;
use Fiscal\Entities\Incitemp;
use Fiscal\Entities\Itemproimpos;
use Fiscal\Entities\Mapaapurac;
use Fiscal\Entities\Notafiscal;
use Fiscal\Entities\Trabimposnftot;
use Fiscal\Entities\Trabitproimpos;
use Fiscal\Logic\Impostos\GeradorImposto;
use Fiscal\Logic\NotaFiscal\Entrada\NFEntrada;
use Fiscal\Logic\NotaFiscal\Lancamento\LancamentoNF;
use Geral\Entities\Cliente;
use Geral\Entities\Filial;
use Geral\Entities\Fornecedor;
use Geral\Entities\Inscrestad;
use Geral\Entities\Itemproduto;
use Geral\Entities\Lanccont;
use Geral\Entities\Paramfilial;
use Geral\Entities\Paramgeral;
use Geral\Entities\Produto;
use Geral\Entities\Tipooperac;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Indeterminado\Entities\Filtpfaturfis;
use Indeterminado\Entities\Impobrigtpoper;
use Setforms2\Shared\Registry;

class AuditoriaDocumentos
{
    private static $arrayMsgErros = array(
        'RECESTOQUE' => 'Índice financeiro parametrizado na personalização do sistema não possui valor cadastrado.',
        'IMPOBR'     => 'Operação bloqueada. Nota fiscal contem itens sem incidência de impostos obrigatórios.',
        'CST_IMP'    => 'Operação bloqueada. Nota fiscal possui impostos sem CST.',
        'ERRO_CST'   => 'Operação bloqueada. Nota fiscal possui itens sem CST.',
        'CFOP_IMP'   => 'Operação bloqueada. Nota fiscal possui impostos sem CFOP.',
        'ERRO_CFOP'  => 'Operação bloqueada. Nota fiscal possui itens sem CFOP.',
        'ALTERA_FIN' => 'Operação bloqueada. Auditoria na nota fiscal alteraria o valor do vencimento.',
        'BLOQ'       => 'Operação bloqueada. Nota está sendo usada por outro operador.',
        'LANCAPUR'   => 'Operação bloqueada. A nota fiscal está atrelada a um mapa apurado bloqueado.',
        'DTLANC'     => 'Operação bloqueada. Data de emissão da nota fiscal não está dentro do período de lançamento contábil.',
        'MPLUCPRE'   => 'Operação bloqueada. Nota fiscal está relacionada a um mapa de lucro presumido que está bloqueado.',
        'IMP_XML'    => 'Operação bloqueada! Nota fiscal lançada via importação de arquivo com extensão XML.',
        'NCONTAB'    => 'Operação bloqueada. Erro na contabilização.'
    );

    public static function aprovAuditoria(Notafiscal $notafiscal, $cdgrupofisc)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            $nrorg      = $notafiscal->getNrorg();
            $filial     = $notafiscal->getCdfilial();
            $nrlanctonf = $notafiscal->getNrlanctonf();
            $nrnotafisc = $notafiscal->getNrnotafisc();
            $erros = array();

            $dadosNF = Notafiscal::getRepository()->dadosNF($filial, $nrlanctonf, $nrorg);

            $bloqueio = Impostonf::getRepository()->consultaBloqueio($dadosNF['CDFILIAL'], $dadosNF['NRLANCTONF'], $nrorg);

            if ((!empty($bloqueio['NRSEQUMAPAPU'])) || (!empty($bloqueio['NRSEQMAPOIMP']))) {
                $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['LANCAPUR']);
            }

            if (($dadosNF['IDSTATUS'][0] == 'A') || ($dadosNF['IDSTATUS'][0] == 'E')) {
                if ($dadosNF['IDENTRSAIDOP'] == 'E') {
                    $data = $dadosNF['DTENTRSAID'];
                } else {
                    $data = $dadosNF['DTEMISSAO'];
                }
                $verifDtLanc = Contopectbl::getRepository()->verificaDataLancamento($dadosNF['CDEMPRESA'], $data, $nrorg);
                if (empty($verifDtLanc)) {
                    $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['DTLANC']);
                }
            }

            /**
             * Validação para bloquear a Auditoria de Notas que estão relacionadas a um Mapa
             * de Lucro presumido que está bloqueado(somente se for "fechar documento").
             */
            if (!empty($dadosNF['NRSEQUMAPAPUNF'])) {
                $mapaLrPresu = Mapaapurac::getRepository()->consultaMapaLrPresu($dadosNF['NRSEQUMAPAPUNF'], $nrorg);
                if (!empty($mapaLrPresu)) {
                    $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['MPLUCPRE']);
                }
            }

            /**
             * Validação para não auditar nota fiscal importada via XML caso o parâmetro de
             * NF igual xml esteja marcado.
             */
            $paramgeral = Paramgeral::findOneByNrorg(Ambiente::getNrorg());
            if ($paramgeral->getIdgernfigualxml() == 'S') {
                if ($dadosNF['IDNFGERADAVIAXML'] == 'S') {
                    $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['IMP_XML']);
                }
            }

            /**
             * Validação para não auditar nota fiscal importada via xml caso o parâmetro de
             * NF igual xml esteja marcado.
             */
            if ($dadosNF['IDENTRSAIDOP'] == 'E') {
                //acmUpdNotaFiscal
                self::atualizaCfop($filial, $nrlanctonf, $nrorg, null, null);

                //Procedure de Contabilização(Entrada)
                $nrlancctb = Contabilizacao::geraCtbDfeContab($dadosNF['CDFILIAL'], $dadosNF['NRLANCTONF']);

                if (!empty($nrlancctb)) {
                    /** @var Erroctbtemp $erroctbtemp */
                    $erroctbtemp = Erroctbtemp::findByNrorgAndCdempresaAndNrlancctbtemp($nrorg, $dadosNF['CDEEMPRESA'], $nrlancctb);
                    if (!empty($erroctbtemp)) {
                        $lancctbtemp = Lancctbtemp::getRepository()->getLancctbtemp($dadosNF['CDEEMPRESA'], $dadosNF['NRLANCTONF']);
                        $ratectbtemp = Lancctbtemp::getRepository()->getRatectbtemp($dadosNF['CDEEMPRESA'], $dadosNF['NRLANCTONF'], '', 'T');

                        //@todo Retorno ErroCtbTemp, associar $lancctbtemp e $ratectbtemp
                        $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['LANCCTB_INV']);
                    }
                }

                $erros = self::fechaNotaFiscal($nrorg, $filial, $nrlanctonf, $dadosNF, $cdgrupofisc, $nrnotafisc, $erros);

            } else {
                /**
                 * Se não for uma nota fiscal transmitida, permite fechar novamente.
                 */
                if (($dadosNF['IDSTATUSNFE'] == 'P') || ($dadosNF['IDSTATUSNFE'] == 'R')) {
                    //acmUpdNotaFiscal
                    self::atualizaCfop($filial, $nrlanctonf, $nrorg, null, null);

                    //Procedure de Contabilização(Saida)
                    $nrlancctb = Contabilizacao::geraCtbDfsContab($dadosNF['CDFILIAL'], $dadosNF['NRLANCTONF']);

                    if (!empty($nrlancctb)) {
                        /** @var Erroctbtemp $erroctbtemp */
                        $erroctbtemp = Erroctbtemp::findByNrorgAndCdempresaAndNrlancctbtemp($nrorg, $dadosNF['CDEEMPRESA'], $nrlancctb);
                        if (!empty($erroctbtemp)) {
                            $lancctbtemp = Lancctbtemp::getRepository()->getLancctbtemp($dadosNF['CDEEMPRESA'], $dadosNF['NRLANCTONF']);
                            $ratectbtemp = Lancctbtemp::getRepository()->getRatectbtemp($dadosNF['CDEEMPRESA'], $dadosNF['NRLANCTONF'], '', 'T');

                            //@todo Retorno ErroCtbTemp, associar $lancctbtemp e $ratectbtemp
                            $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['LANCCTB_INV']);
                        }
                    }

                    $dbTotalNFAnt = $dadosNF['VRNOTAFISC'];

                    $rateioNF = LancamentoNF::geracaoRateioAnterior($notafiscal);

                    $dbTotalRetFontAnt = Imposnftot::getRepository()->vrTotalRetFont($filial, $nrlanctonf, $nrorg);

                    $dbTotalNFNovo = ProceduresNF::callProcedureImpostoSaida($notafiscal);
                    Registry::getEntityManager()->refresh($notafiscal);
                    self::updateValorNota($dadosNF['CDFILIAL'], $dadosNF['NRLANCTONF'], $dadosNF['NRORG'], $dbTotalNFNovo);

                    $dbTotalRetFontPos = Imposnftot::getRepository()->vrTotalRetFont($filial, $nrlanctonf, $nrorg);

                    //adqImpObrg
                    $impostosObrigatorios = Impobrigtpoper::getRepository()->getImpostosObrigatorios(
                        $dadosNF['IDENTRSAIDOP'],
                        $dadosNF['CDTIPOOPER'], $dadosNF['NRORG'],
                        $dadosNF['IDENTRSAIDOP'] == 'E' ? $dadosNF['CDFILIRECE'] : $dadosNF['CDFILIEMIT']
                    );

                    if (!empty($impostosObrigatorios)) {
                        $itemproduto = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($filial, $nrlanctonf, $nrorg);
                        /** @var Itemproduto $item */
                        foreach ($itemproduto as $item) {
                            foreach ($impostosObrigatorios as $impos) {
                                $itemproimpos = Itemproimpos::findOneByCdfilialAndNrlanctonfAndNrseqitprodAndNrorgAndCdimposto($filial, $nrlanctonf, $item->getNrseqitprod(), $nrorg, $impos['CDIMPOSTO']);
                                if (!$itemproimpos instanceof Itemproimpos) {
                                    $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['IMPOBR']);
                                }
                            }
                        }
                    }

                    $adqConsParFisc = Tipooperac::getRepository()->consParFisc($dadosNF['CDTIPOOPER'], $dadosNF['IDENTRSAIDOP'], $dadosNF['NRORG']);

                    if ($adqConsParFisc['IDCONSPARFIS'] == 'S') {
                        $cst = Itemproimpos::getRepository()->verificaCST($filial, $nrlanctonf, $nrorg);
                        if (!empty($cst)) {
                            $locate = array_filter(
                                $cst, function ($arr) {
                                    return ($arr['CDSITUTRIBIT'] == 'XXX');
                                }
                            );
                            if ($locate) {
                                $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['CST_IMP']);
                            }
                        } else {
                            $cstIt = Itemproimpos::getRepository()->verificaCSTit($filial, $nrlanctonf, $nrorg);
                            if (!empty($cstIt)) {
                                $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['ERRO_CST']);
                            }
                        }
                    }

                    if ($adqConsParFisc['IDCONSPARFIS'] == 'S') {
                        $verificaCFOP = Itemproimpos::getRepository()->verificaCFOP($filial, $nrlanctonf, $nrorg);
                        if (!empty($verificaCFOP)) {
                            $locate = array_filter(
                                $verificaCFOP, function ($arr) {
                                    return ($arr['CDFISCOPERIT'] == 'XXX');
                                }
                            );
                            if ($locate) {
                                $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['CFOP_IMP']);
                            }
                        } else {
                            $verificaCFOPit = Itemproimpos::getRepository()->verificaCFOPit($filial, $nrlanctonf, $nrorg);
                            if (!empty($verificaCFOPit)) {
                                $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['ERRO_CFOP']);
                            }
                        }
                    }

                    LancamentoNF::geraRateioPadrao($notafiscal);

                    if ((abs($dbTotalNFNovo - $dbTotalNFAnt) > 0.02) || ($dbTotalRetFontAnt != $dbTotalRetFontPos)) {
                        if ($dadosNF['IDUTILVENC'] != 'N') {
                            $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['ALTERA_FIN']);
                        }
                    }
                }
            }

            if(empty($erros)){
                $connection->commit();
                return true;
            }
            else{
                $connection->rollback();
                return $erros;
            }
        } catch (\Exception $e) {
            $connection->rollback();
            tk_warning($e->getMessage());
            return false;
        }
    }

    public static function atualizaCfop($filial, $nrlanctonf, $organizacao, $cdfisopentf, $dscdfisopenf)
    {
        /** @var Notafiscal $notafiscal */
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $nrlanctonf, $organizacao);

        $notafiscal->setCdfisopentf($cdfisopentf);
        $notafiscal->setDscdfisopenf($dscdfisopenf);
        $notafiscal->persist();

        Registry::getEntityManager()->flush();
    }

    public static function fechaNotaFiscal($nrorg, $filial, $nrlanctonf, $dadosNF, $cdgrupofisc, $nrnotafisc, $erros)
    {
        /** @var Paramgeral $paramgeral */
        $paramgeral = Paramgeral::findOneByNrorg(Ambiente::getNrorg());
        /** @var Notafiscal $notafiscal */
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $nrlanctonf, $nrorg);

        if ($paramgeral->getIdreaudparfisc() == 'S') {
            NFEntrada::deleteImpostoNotaTotal($filial, $nrlanctonf, $nrorg);

            //adqImpoInfoMan[BACK]
            $itemproduto = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($filial, $nrlanctonf, $nrorg);
            /** @var Itemproduto $item */
            foreach ($itemproduto as $item) {

                /** @var Itemproimpos $itemproimpos */
                $itemproimpos = Itemproimpos::findByCdfilialAndNrlanctonfAndNrseqitprodAndIdvalorinfitAndNrorg($filial, $nrlanctonf, $item->getNrseqitprod(), 'S', $nrorg);

                /** @var Produto $produto */
                $produto = Produto::findOneByCdprodutoAndNrorg($item->getCdproduto(), $item->getNrorg());

                $opInEst = Inscrestad::getRepository()->opInEst($dadosNF['CDFILIEMIT'], $dadosNF['CDFORNECED'], $dadosNF['CDCLIENTE'], $dadosNF['CDINSCESTA'], $dadosNF['CDEMPRESA'], $nrorg);

                ProceduresNF::callProcedureImpostoEntradaProduto($notafiscal, $cdgrupofisc, $produto->getCdarvprod(), $item->getVrtotaitem() - $item->getVrdescitem(), empty($opInEst) ? 'S' : 'N');
                Registry::getEntityManager()->refresh($notafiscal);

                //adqAliqUniFis
                /** @var Incitemp $incitemp */
                $incitemp = Incitemp::getRepository()->getImpostosTemporarios($filial, $nrlanctonf, $nrorg, null);

                //adqImpObrg
                /** @var Impobrigtpoper $impostosObrigatorios */
                $impostosObrigatorios = Impobrigtpoper::getRepository()->getImpostosObrigatorios($dadosNF['IDENTRSAIDOP'], $dadosNF['CDTIPOOPER'], $nrorg, $dadosNF['IDENTRSAIDOP'] == 'E' ? $dadosNF['CDFILIRECE'] : $dadosNF['CDFILIEMIT']);

                if (!empty($impostosObrigatorios)) {
                    foreach ($impostosObrigatorios as $impObrig) {
                        /** @var Incitemp $incitemp */
                        $aliqunifis = Incitemp::getRepository()->getImpostosTemporarios($filial, $nrlanctonf, $nrorg, $impObrig['CDIMPOSTO']);
                        if (empty($aliqunifis)) {
                            $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['IMPOBR']);
                        }
                    }
                }

                if (!empty($incitemp)) {
                    foreach ($incitemp as $imptemp) {
                        if ($imptemp['IDPRODNOTA'] == 'P') {
                            NFEntrada::deleteImpostoTemporarioProduto($filial, $nrlanctonf, $nrorg, $item->getNrseqitprod(), $imptemp['CDIMPOSTO'], '1');

                            if (!empty($itemproimpos)) {
                                //locate
                                foreach ($itemproimpos as $itemproimp) {
                                    /** @var Itemproimpos $itemproimpos */
                                    $trabitproimpos = Itemproimpos::findByCdfilialAndNrlanctonfAndNrseqitprodAndIdvalorinfitAndNrorgAndCdimposto($filial, $nrlanctonf, $item->getNrseqitprod(), 'S', $nrorg, $itemproimp['CDIMPOSTO']);
                                    if (!empty($trabitproimpos)) {
                                        Trabitproimpos::getRepository()->insTrabItProImpos($filial, $nrlanctonf, $item->getNrseqitprod(), $incitemp['CDIMPOSTO'], '1', $incitemp['VRALIQUOTA'], 0, 0, 0, 0, $itemproimpos['VRIMPOSTOIT'], $itemproimpos['IDVALORINFIT'], $nrorg);
                                    } else {
                                        Trabitproimpos::getRepository()->insTrabItProImpos($filial, $nrlanctonf, $item->getNrseqitprod(), $incitemp['CDIMPOSTO'], '1', $incitemp['VRALIQUOTA'], 0, 0, 0, 0, 0, 'N', $nrorg);
                                    }
                                }
                            } else {
                                Trabitproimpos::getRepository()->insTrabItProImpos($filial, $nrlanctonf, $item->getNrseqitprod(), $imptemp['CDIMPOSTO'], '1', $imptemp['VRALIQUOTA'], 0, 0, 0, 0, 0, 'N', $nrorg);
                            }
                        } else {
                            NFEntrada::deleteImpostoNotaTotal($filial, $nrlanctonf, $nrorg, $imptemp['CDIMPOSTO']);
                            self::insertImpostoNF($filial, $nrlanctonf, $imptemp['CDIMPOSTO'], '0', $nrorg);
                        }
                        NFEntrada::deletaImpostoTemporario($filial, $nrlanctonf, $nrorg, $imptemp['CDIMPOSTO']);
                    }
                }
            }

            //Totalização dos Impostos na TRABIMPOSNFTOT
            $impostosTemporarios = Trabitproimpos::getRepository()->impostosTemporarios($filial, $nrlanctonf, $nrorg);
            foreach ($impostosTemporarios as $impostoTemp) {
                self::insertImpostoNF($filial, $nrlanctonf, $impostoTemp['CDIMPOSTOIT'], $impostoTemp['VRTOTAIMPO'], $nrorg);
            }

        } else {
            Trabitproimpos::getRepository()->insertImpostosTemporariosDaItemproimpos($filial, $nrlanctonf, $nrorg);
            Trabimposnftot::getRepository()->insertImpostosTemporariosDaImposnftot($filial, $nrlanctonf, $nrorg);
        }

        $dbTotalNFAnt = $dadosNF['VRNOTAFISC'];
        //adqVrTotRetFont
        $dbTotalRetFontAnt = Imposnftot::getRepository()->vrTotalRetFont($filial, $nrlanctonf, $nrorg);

        $rateioNF = LancamentoNF::geracaoRateioAnterior($notafiscal);

        if ($notafiscal->getIdentrsaidop() == Notafiscal::NOTA_FISCAL_ENTRADA) {
            $dbTotalNFNovo = GeradorImposto::callProcedureImposto($notafiscal);
            Registry::getEntityManager()->refresh($notafiscal);
            GeradorImposto::callProcedureGravaImposto($notafiscal);
        } else {
            $dbTotalNFNovo = ProceduresNF::callProcedureImpostoSaida($notafiscal);
            Registry::getEntityManager()->refresh($notafiscal);
            self::updateValorNota($dadosNF['CDFILIAL'], $dadosNF['NRLANCTONF'], $dadosNF['NRORG'], $dbTotalNFNovo);
        }

        LancamentoNF::geraRateioPadrao($notafiscal);

        $dbTotalRetFontPos = Imposnftot::getRepository()->vrTotalRetFont($filial, $nrlanctonf, $nrorg);

        $adqConsParFisc = Tipooperac::getRepository()->consParFisc($dadosNF['CDTIPOOPER'], $dadosNF['IDENTRSAIDOP'], $dadosNF['NRORG']);

        if ($adqConsParFisc['IDCONSPARFIS'] == 'S') {
            $cst = Itemproimpos::getRepository()->verificaCST($filial, $nrlanctonf, $nrorg);
            if (!empty($cst)) {
                $locate = array_filter(
                    $cst, function ($arr) {
                        return ($arr['CDSITUTRIBIT'] == 'XXX');
                    }
                );
                if ($locate) {
                    $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['CST_IMP']);
                }
            } else {
                $cstIt = Itemproimpos::getRepository()->verificaCSTit($filial, $nrlanctonf, $nrorg);
                if (!empty($cstIt)) {
                    $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['ERRO_CST']);
                }
            }

            //adqVerifCFOP
            $cfop = Itemproimpos::getRepository()->verificaCFOP($filial, $nrlanctonf, $nrorg);
            if (!empty($cfop)) {
                $locate = array_filter(
                    $cst, function ($arr) {
                        return ($arr['CDFISCOPERIT'] == 'XXX');
                    }
                );
                if ($locate) {
                    $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['CFOP_IMP']);
                }
            } else {
                $cfopIt = Itemproimpos::getRepository()->verificaCFOPit($filial, $nrlanctonf, $nrorg);
                if (!empty($cfopIt)) {
                    $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['ERRO_CFOP']);
                }
            }

        }

        if ((abs($dbTotalNFNovo['P_VRNOTAFISC'] - $dbTotalNFAnt) > 0.02) || ($dbTotalRetFontAnt != $dbTotalRetFontPos)) {
            if ($dadosNF['NFIDUTILVENC'] != 'N') {
                $erros[] = array('CDFILIAL' => $filial, 'NRNOTAFISC' => $nrnotafisc, 'MENSAGEM' => self::$arrayMsgErros['ALTERA_FIN']);
            }
        }

        self::AtualizaEstoque($dadosNF['CDTIPOOPER'], $filial, $nrlanctonf, $nrorg, $dadosNF['VRPRODUTOS'], $dadosNF['CDFILIRECE'], $dadosNF['DTENTRSAID']);
        return $erros;
    }

    public static function insertImpostoNF($cdfilial, $nrlanctonf, $cdimpostot, $vrimpostot, $nrorg)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {

            /** @var Trabimposnftot $trabimposnftot */
            $trabimposnftot = new Trabimposnftot();

            $trabimposnftot->setCdfilimptot($cdfilial);
            $trabimposnftot->setNrlancimptot($nrlanctonf);
            $trabimposnftot->setCdimpostot($cdimpostot);
            $trabimposnftot->setVrimpostot($vrimpostot);
            $trabimposnftot->setVrbcalimptot('0');
            $trabimposnftot->setIddifaliqcusptn('N');
            $trabimposnftot->setNrorg($nrorg);
            $trabimposnftot->setDtinclusao(DateUtil::getDataAtual());
            $trabimposnftot->setDtultatu(DateUtil::getDataAtual());
            $trabimposnftot->setCdoperinclusao(Ambiente::getCdOperador());
            $trabimposnftot->setCdoperultatu(Ambiente::getCdOperador());

            $trabimposnftot->persist();

            Registry::getEntityManager()->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollback();
            tk_abort($e->getMessage());
        }
    }

    /**
     * Altera o valor da nota fiscal
     * @param $filial
     * @param $lancamento
     * @param $organizacao
     * @param $valor
     * @return void
     */
    public static function updateValorNota($filial, $lancamento, $organizacao, $valor)
    {
        /** @var Notafiscal $notafiscal */
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

        $notafiscal->setVrnotafisc($valor);
        $notafiscal->persist();

        Registry::getEntityManager()->flush();
    }

    public static function AtualizaEstoque($cdtipooper, $cdfilial, $nrlanctonf, $nrorg, $vrprodutos, $cdfilirece, $dtentrsaid)
    {
        if ($cdtipooper != '11') {
            /** @var Lanctoestoq $lanctoestoq */
            $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlanctonfAndNrorg($cdfilial, $nrlanctonf, $nrorg);
            CalculaCustoLiquido::CalcularCustoLiquido($cdfilial, $lanctoestoq->getNrlancestq(), $vrprodutos);

            /** @var Paramfilial $adqFilial */
            $adqFilial = Paramfilial::findOneByCdfilialAndNrorg($cdfilirece, $nrorg);;

            if ($adqFilial->getIdctrlestq() != 'N' && $adqFilial->getIdestonline() != 'N') {
                $dataInicial = $dtentrsaid;
                $paramfilial = Paramfilial::findOneByCdfilialAndNrorg($cdfilial, $nrorg);
                if (DateUtil::comparaDatas(DateUtil::getDataDeString($dtentrsaid, DateUtil::FORMATO_BRASILEIRO_DATAHORA), $paramfilial->getDataProcessamento(), DateUtil::DEPOIS)) {
                    $dataFinal = $dtentrsaid;
                } else {
                    $dataFinal = $paramfilial->getDataProcessamento()->format(DateUtil::FORMATO_BRASILEIRO);
                }

                AtualizadorDeEstoque::executarProcedureAtualizaEstoque($lanctoestoq, $dataFinal, $dataInicial);
            }
        }
    }

    public static function consultaGrupoFiscal($nrorg, $idorigdestnf, $cdorigdest, $cdfilifatur, $identrsaidop, $cdtipooper, $cdgrupofisc)
    {
        if ($idorigdestnf == 'F') {
            /** @var Fornecedor $fornecedor */
            $fornecedor = Fornecedor::findOneByNrorgAndCdforneced($nrorg, $cdorigdest);
            $auxiliar   = $fornecedor->getCdgrupofisc();
        } elseif ($idorigdestnf == 'C') {
            /** @var Cliente $cliente */
            $cliente  = Cliente::findOneByNrorgAndCdcliente($nrorg, $cdorigdest);
            $auxiliar = $cliente->getCdgrupofisc();
        } else {
            /** @var Filial $filial */
            $filial   = Filial::findOneByNrorgAndCdfilial($nrorg, $cdorigdest);
            $auxiliar = $filial->getCdgrupofisc();
        }

        if ((empty($auxiliar)) && (!empty($cdfilifatur))) {
            $auxiliar = $cdgrupofisc;
        }

        if (empty($auxiliar)) {
            /** @var Tipooperac $tipooperac */
            $tipooperac = Tipooperac::findOneByNrorgAndCdtipooperAndIdentrsaidop($nrorg, $cdtipooper, $identrsaidop);
            $auxiliar   = $tipooperac->getCdgrupofisc();
        }

        if (empty($auxiliar)) {
            $auxiliar = Filtpfaturfis::getRepository()->getGrupoFiscFatur($cdfilifatur);
            $auxiliar = $auxiliar[0];
        }

        return $auxiliar;
    }

    public static function insertImpostoProduto($cdfilimpoit, $nrlancimpoit, $nrseqimpoit, $cdimpostoit, $nrseqitpro,
                                                $vraliqimpoit, $vrimposto, $idvalorinfo, $nrorg)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {

            $trabitproimpos = new Trabitproimpos();

            // PK_ITEMPROIMPOS
            $trabitproimpos->setCdfilimpoit($cdfilimpoit);
            $trabitproimpos->setNrlancimpoit($nrlancimpoit);
            $trabitproimpos->setNrseqimpoit($nrseqimpoit);
            $trabitproimpos->setCdimpostoit($cdimpostoit);
            $trabitproimpos->setNrseqitpro($nrseqitpro);
            $trabitproimpos->setVraliqimpoit($vraliqimpoit);
            $trabitproimpos->setVrimposit($vrimposto);
            $trabitproimpos->setIdvalorinfo($idvalorinfo);
            $trabitproimpos->setVrcontimpoit('0');
            $trabitproimpos->setVrredbcimpoit('0');
            $trabitproimpos->setVrbcalcimpoit('0');
            $trabitproimpos->setVrpdedimpoit('0');
            $trabitproimpos->setNrorg($nrorg);
            $trabitproimpos->setCdoperinclusao(Ambiente::getCdOperador());
            $trabitproimpos->setDtinclusao(DateUtil::getDataDeString(now(), DateUtil::FORMATO_BRASILEIRO_DATAHORA));
            $trabitproimpos->setCdoperultatu(Ambiente::getCdOperador());
            $trabitproimpos->setDtultatu(DateUtil::getDataDeString(now(), DateUtil::FORMATO_BRASILEIRO_DATAHORA));

            $trabitproimpos->persist();

            Registry::getEntityManager()->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            tk_abort($e->getMessage());
        }
    }

    public static function updateFisopeNF($filial, $nrlanctonf, $organizacao)
    {
        /** @var Notafiscal $notafiscal */
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $nrlanctonf, $organizacao);

        $notafiscal->setCdfisopentf(null);
        $notafiscal->setDscdfisopenf(null);
        $notafiscal->persist();

        Registry::getEntityManager()->flush();
    }

    public static function deleteAjusteLancCont($cdempresa, $nrlanccont, $nrorg)
    {
        try {
            /** @var $ajuste Ajctpagar */
            $ajuste = Lanccont::findByCdempresaAndNrlanccontAndNrorg($cdempresa, $nrlanccont, $nrorg);
            $ajuste->delete();


        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateAjusteCtPg($cdempresa, $nrlctbajpag, $filial, $nrlanctonf, $nrseqvenpag, $organizacao)
    {
        /** @var Ajctpagar $ajctpagar */
        $ajctpagar = Ajctpagar::findOneByCdfilialAndNrlanctonfAndNrseqvenpagAndNrorg($filial, $nrlanctonf, $nrseqvenpag, $organizacao);

        $ajctpagar->setCdempresa($cdempresa);
        $ajctpagar->setNrlctbajpag($nrlctbajpag);
        $ajctpagar->persist();

        Registry::getEntityManager()->flush();
    }

    public static function updateLancContNF($lanccont, $filial, $nrlanctonf, $organizacao)
    {
        /** @var Notafiscal $notafiscal */
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $nrlanctonf, $organizacao);

        $notafiscal->setNrlanctbdoc($lanccont);
        $notafiscal->persist();

        Registry::getEntityManager()->flush();
    }

    public static function updateNotaFiscal($filial, $nrlanctonf, $organizacao, $operacao, $dtemissao, $dtsaidemis, $nrnotafisc, $lancctb)
    {
        /** @var Notafiscal $notafiscal */
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $nrlanctonf, $organizacao);

        $notafiscal->setIdstatus($operacao);
        $notafiscal->setDtemissao($dtemissao);
        $notafiscal->setDtlancamennf($dtsaidemis);
        $notafiscal->setNrnotafisc($nrnotafisc);
        $notafiscal->setNrlanctbdoc($lancctb);
        $notafiscal->persist();

        Registry::getEntityManager()->flush();
    }
}