/*
@C:\web_649\sql\end_conv\oracle\0046_EXPSTPO_RECALCULA_PEDIDO_EXP.sql
 DESCRIO:
 Procedure para atualizar a situao dos itens/pedido a partir da quantidade expedida na notafiscal

PARMETROS

 P_CDFILIAL    - CDDIGO DA FILIAL
 P_NRLANCTONF  - NUMERO DO LANAMENTO DA NOTAFISCAL

 RAISE_APPLICATION_ERROR(-20501, '' , TRUE);
*/
create or replace PROCEDURE RECALCULA_PEDIDO_EXP(P_CDFILIAL IN VARCHAR2,
                                                 P_NRLANCTONF IN VARCHAR2)
IS
    V_IDSITITEMPED  ITEMPEDIDO.IDSITITEMPED%TYPE;
    V_QTEXPITEMPED  ITEMPEDIDO.QTEXPITEMPED%TYPE;
    V_QTITEMPED     ITEMPEDIDO.QTITEMPED%TYPE;
    V_VRPELIQEXP    PAREXPEDICAO.VRPELIQEXP%TYPE;
    V_NRITENS       INTEGER;
    V_CDOPERADOR    OPERADOR.CDOPERADOR%TYPE;
    V_VRFATOCONV    PRODUTO.VRFATOCONV%TYPE;
    V_QTEXPEDGERAL  ITEMPRODUTO.QTITEMPROD%TYPE;
    V_QTITEMPROD    ITEMPRODUTO.QTITEMPROD%TYPE;
    V_IDGERPRODEST  PAREXPEDICAO.IDGERPRODEST%TYPE;
    V_CDFILIALPED   ITEMPRODUTO.CDFILIALPED%TYPE;
    V_NRPEDIDO      ITEMPRODUTO.NRPEDIDO%TYPE;
    V_NRITEMPED     ITEMPRODUTO.NRITEMPED%TYPE;

BEGIN    
    
    FOR C_ITEMPRODUTO IN(
        SELECT I.CDFILIAL, I.NRLANCTONF,I.CDPRODUTO,I.NRSEQITPROD,
               I.CDFILIALPED, I.NRPEDIDO, I.NRITEMPED,
               SUM(I.QTITEMPROD) QTITEMPROD
         FROM NOTAFISCAL NF, ITEMPRODUTO I
        WHERE NF.CDFILIAL   = I.CDFILIAL
          AND NF.NRLANCTONF = I.NRLANCTONF
          AND I.CDFILIAL    = P_CDFILIAL
          AND I.NRLANCTONF  = P_NRLANCTONF
          GROUP BY
          I.CDFILIAL, I.NRLANCTONF, I.CDPRODUTO, I.NRSEQITPROD,
          I.CDFILIALPED, I.NRPEDIDO, I.NRITEMPED
    )LOOP
    
        IF ((C_ITEMPRODUTO.CDFILIALPED IS NULL) AND (C_ITEMPRODUTO.NRPEDIDO IS NULL) AND (C_ITEMPRODUTO.NRITEMPED IS NULL)) THEN
            /*Verifico se existe pedido relacionado ao item da nota (com outro lote ou NRSEQITPROD, por exemplo)*/  
            BEGIN
                SELECT MAX(I.CDFILIALPED), MAX(I.NRPEDIDO), MAX(I.NRITEMPED)
                  INTO V_CDFILIALPED , V_NRPEDIDO , V_NRITEMPED
                  FROM NOTAFISCAL NF, ITEMPRODUTO I
                 WHERE NF.CDFILIAL   = I.CDFILIAL
                   AND NF.NRLANCTONF = I.NRLANCTONF
                   AND I.CDFILIAL    = P_CDFILIAL
                   AND I.NRLANCTONF  = P_NRLANCTONF
                   AND I.CDPRODUTO   = C_ITEMPRODUTO.CDPRODUTO;
            EXCEPTION
            WHEN NO_DATA_FOUND THEN
                V_CDFILIALPED := NULL;
                V_NRPEDIDO    := NULL;
                V_NRITEMPED   := NULL;
            END;
            
            IF ((V_CDFILIALPED IS NULL) OR (V_NRPEDIDO IS NULL) OR (V_NRITEMPED IS NULL)) THEN
                /*Se no encontrou V_CDFILIALPED,V_NRPEDIDO e V_NRITEMPED na query anterior, verifico se existe estes dados na tabela ITEMPEDIDONF*/
                BEGIN
                    SELECT MAX(I.CDFILIAL), MAX(I.NRPEDIDO), MAX(I.NRITEMPED)
                      INTO V_CDFILIALPED , V_NRPEDIDO , V_NRITEMPED
                      FROM NOTAFISCAL NF, ITEMPEDIDONF I
                     WHERE NF.CDFILIAL   = I.CDFILIAL
                       AND NF.NRLANCTONF = I.NRLANCTONF
                       AND I.CDFILIAL    = P_CDFILIAL
                       AND I.NRLANCTONF  = P_NRLANCTONF
                       AND I.CDPRODUTO   = C_ITEMPRODUTO.CDPRODUTO;
                EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    V_CDFILIALPED := NULL;
                    V_NRPEDIDO    := NULL;
                    V_NRITEMPED   := NULL;
                END;
            END IF;
    
            IF ((V_CDFILIALPED IS NOT NULL) AND (V_NRPEDIDO IS NOT NULL) AND (V_NRITEMPED IS NOT NULL)) THEN
                IF ((C_ITEMPRODUTO.CDFILIALPED  IS NULL) AND (C_ITEMPRODUTO.NRPEDIDO IS NULL) AND (C_ITEMPRODUTO.NRITEMPED    IS NULL)) THEN
                   /*Se o cursor C_ITEMPRODUTO nao trouxe dados do pedido, mas existe pedido vinculado ao item da NF (V_CDFILIALPED, V_NRPEDIDO e V_NRITEMPED)
                     pode indicar que a NF teve este item inserido manualmente, e neste caso os dados do pedido sao updatados neste item tambm*/
                    UPDATE ITEMPRODUTO
                       SET NRPEDIDO    = V_NRPEDIDO, NRITEMPED = V_NRITEMPED, CDFILIALPED = V_CDFILIALPED
                     WHERE CDFILIAL    = C_ITEMPRODUTO.CDFILIAL
                       AND NRLANCTONF  = C_ITEMPRODUTO.NRLANCTONF
                       AND NRSEQITPROD = C_ITEMPRODUTO.NRSEQITPROD
                       AND CDPRODUTO   = C_ITEMPRODUTO.CDPRODUTO;
                END IF;
            END IF;
        END IF;
    END LOOP;
    
    FOR C_PRODCONS IN(
        SELECT I.CDFILIAL, I.NRLANCTONF,I.CDPRODUTO,
               I.CDFILIALPED, I.NRPEDIDO, I.NRITEMPED,
               SUM(I.QTITEMPROD) QTITEMPROD
         FROM NOTAFISCAL NF, ITEMPRODUTO I
        WHERE NF.CDFILIAL   = I.CDFILIAL
          AND NF.NRLANCTONF = I.NRLANCTONF
          AND I.CDFILIAL    = P_CDFILIAL
          AND I.NRLANCTONF  = P_NRLANCTONF
          GROUP BY
          I.CDFILIAL, I.NRLANCTONF, I.CDPRODUTO,
          I.CDFILIALPED, I.NRPEDIDO, I.NRITEMPED
    )LOOP

    IF ((C_PRODCONS.CDFILIALPED IS NOT NULL) AND (C_PRODCONS.NRPEDIDO IS NOT NULL) AND (C_PRODCONS.NRITEMPED IS NOT NULL)) THEN
        SELECT MAX(I.IDSITITEMPED),MAX(I.QTITEMPED),MAX(P.VRPELIQEXP)
          INTO V_IDSITITEMPED,V_QTITEMPED,V_VRPELIQEXP
          FROM ITEMPEDIDO I, PAREXPEDICAO P
         WHERE ((P_CDFILIAL    = I.CDFILIAL)    AND
                (C_PRODCONS.NRPEDIDO    = I.NRPEDIDO)    AND
                (C_PRODCONS.NRITEMPED   = I.NRITEMPED)   AND
                (C_PRODCONS.CDFILIALPED = P.CDFILIAL));
                
        
        V_QTEXPITEMPED := C_PRODCONS.QTITEMPROD;

        /*Verifica se existe algum item do pedido vinculado a NF que nao esta liquidado porque nesse caso
        o status do PEDIDO nao pode ser alteardo para LIQUIDADO*/
        BEGIN
            SELECT  COUNT(*)
              INTO V_NRITENS
              FROM ITEMPEDIDO
             WHERE (CDFILIAL     =  C_PRODCONS.CDFILIALPED)
               AND (NRPEDIDO     =  C_PRODCONS.NRPEDIDO)
               AND (NRITEMPED    <> C_PRODCONS.NRITEMPED)
               AND (IDSITITEMPED IN ('1','2','3','7'));
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            V_NRITENS := 0;
        END;

        BEGIN
            SELECT MAX(CDOPERADOR)
            INTO V_CDOPERADOR
            FROM ITEMPEDIDO
            WHERE CDFILIAL = C_PRODCONS.CDFILIALPED AND
                NRPEDIDO   = C_PRODCONS.NRPEDIDO AND
                NRITEMPED  = C_PRODCONS.NRITEMPED;
        EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_CDOPERADOR := '';
        END;

        BEGIN
          SELECT IDGERPRODEST
          INTO V_IDGERPRODEST /*Define se grava o produto  o do PEDIDO ou do ESTOQUE*/
          FROM PAREXPEDICAO
          WHERE CDFILIAL = C_PRODCONS.CDFILIALPED;
        EXCEPTION
          WHEN NO_DATA_FOUND THEN
             V_IDGERPRODEST := '1';
        END;

        /*Traz todas as quantidades do produto do pedido expedido em OUTRAS NF's
        Obs: Considero o QTITEMPROD como valor  EXPEDIDO*/
        BEGIN
            SELECT NVL(SUM(QTITEMPROD),0)
              INTO V_QTEXPEDGERAL
              FROM ITEMPRODUTO
             WHERE CDFILIAL   =  P_CDFILIAL
               AND NRLANCTONF <> P_NRLANCTONF
               AND NRPEDIDO   =  C_PRODCONS.NRPEDIDO
               AND NRITEMPED  =  C_PRODCONS.NRITEMPED;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                V_QTEXPEDGERAL := 0;
        END;

        /*Se o identificador do produto na NF for  PRODUTO DO PEDIDO ...*/
        IF (V_IDGERPRODEST = '1') THEN

            IF ((V_QTEXPITEMPED + V_QTEXPEDGERAL) >= V_QTITEMPED) THEN

                UPDATE ITEMPEDIDO
                  SET  QTEXPITEMPED = V_QTITEMPED
                WHERE (C_PRODCONS.CDFILIALPED = CDFILIAL)   AND
                      (C_PRODCONS.NRPEDIDO    = NRPEDIDO)   AND
                      (C_PRODCONS.NRITEMPED   = NRITEMPED);
            ELSE

                UPDATE ITEMPEDIDO
                SET QTEXPITEMPED = V_QTEXPEDGERAL + V_QTEXPITEMPED
                WHERE (C_PRODCONS.CDFILIALPED = CDFILIAL) AND
                      (C_PRODCONS.NRPEDIDO    = NRPEDIDO) AND
                      (C_PRODCONS.NRITEMPED   = NRITEMPED);

            END IF;
        ELSE
            IF (V_IDGERPRODEST = '2') THEN
                    BEGIN
                        SELECT MAX(A.VRFATOCONV)
                        INTO V_VRFATOCONV
                        FROM PRODUTO A, ITEMPEDIDO I
                        WHERE ((A.CDPRODUTO      = I.CDPRODUTO) AND
                              (C_PRODCONS.CDFILIALPED = I.CDFILIAL ) AND
                              (C_PRODCONS.NRPEDIDO    = I.NRPEDIDO ) AND
                              (C_PRODCONS.NRITEMPED   = I.NRITEMPED));
                    EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                      V_VRFATOCONV := '';
                    END;

                    IF V_VRFATOCONV IS NULL THEN
                        V_VRFATOCONV := 0;
                    END IF;

                    IF (V_VRFATOCONV <> 0) THEN
                        IF (( (V_QTEXPITEMPED / V_VRFATOCONV) + V_QTEXPEDGERAL) >= V_QTITEMPED) THEN

                            UPDATE ITEMPEDIDO
                               SET QTEXPITEMPED = (V_QTITEMPED / V_VRFATOCONV)
                            WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL)   AND
                                   (C_PRODCONS.NRPEDIDO    = NRPEDIDO)   AND
                                   (C_PRODCONS.NRITEMPED   = NRITEMPED));
                        ELSE
                            UPDATE ITEMPEDIDO
                              SET QTEXPITEMPED = (V_QTEXPITEMPED / V_VRFATOCONV) + V_QTEXPEDGERAL 
                            WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL) AND
                                  (C_PRODCONS.NRPEDIDO     = NRPEDIDO) AND
                                  (C_PRODCONS.NRITEMPED    = NRITEMPED));
                        END IF;
                    ELSE

                        IF ((V_QTEXPITEMPED + V_QTEXPEDGERAL) >= V_QTITEMPED) THEN

                          UPDATE ITEMPEDIDO
                          SET QTEXPITEMPED = V_QTITEMPED
                          WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL)    AND
                                 (C_PRODCONS.NRPEDIDO    = NRPEDIDO)    AND
                                 (C_PRODCONS.NRITEMPED   = NRITEMPED));
                        ELSE

                           UPDATE ITEMPEDIDO
                              SET QTEXPITEMPED = (V_QTEXPITEMPED / V_VRFATOCONV) + V_QTEXPEDGERAL 
                            WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL) AND
                                  (C_PRODCONS.NRPEDIDO     = NRPEDIDO) AND
                                  (C_PRODCONS.NRITEMPED    = NRITEMPED));

                        END IF;
                    END IF;
                ELSE
                    UPDATE ITEMPEDIDO
                    SET QTEXPITEMPED = 0
                    WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL)    AND
                           (C_PRODCONS.NRPEDIDO    = NRPEDIDO)    AND
                           (C_PRODCONS.NRITEMPED   = NRITEMPED));
                END IF;
            END IF;

            IF ((V_QTEXPITEMPED + V_QTEXPEDGERAL) >= V_QTITEMPED) THEN

                UPDATE ITEMPEDIDO
                   SET IDSITITEMPED = '4'
                WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL)   AND
                       (C_PRODCONS.NRPEDIDO    = NRPEDIDO)   AND
                       (C_PRODCONS.NRITEMPED   = NRITEMPED));

                IF V_NRITENS  = 0 THEN
                    /*Se nao existe outros itens sem liquidacao, modifica o PEDIDO para LIQUIDADO*/
                    UPDATE PEDIDO
                       SET IDSITPEDIDO = '3'
                     WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL)AND
                           (C_PRODCONS.NRPEDIDO     = NRPEDIDO));
                ELSE
                   /*Como existe outros itens no pedido nao liquidados, modifica o PEDIDO para PARCIALMENTE ATENDIDO*/
                    UPDATE PEDIDO
                       SET IDSITPEDIDO = '6'
                     WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL) AND
                           (C_PRODCONS.NRPEDIDO     = NRPEDIDO));
                END IF;
            ELSE
                UPDATE ITEMPEDIDO
                  SET IDSITITEMPED = '7'
                WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL) AND
                      (C_PRODCONS.NRPEDIDO     = NRPEDIDO) AND
                      (C_PRODCONS.NRITEMPED    = NRITEMPED));

                UPDATE PEDIDO
                  SET IDSITPEDIDO = '6'
                WHERE ((C_PRODCONS.CDFILIALPED = CDFILIAL)AND
                      (C_PRODCONS.NRPEDIDO     = NRPEDIDO));
            END IF;
        END IF;   
    END LOOP;
END;
/
-- VERSAO --
BEGIN ATUVEROBJ('RECALCULA_PEDIDO_EXP','6.49.054'); END;
/
