import {
  Http,
  IDictionary,
  IEventParam,
  Loader,
  Metadata,
} from '@zeedhi/core';
import {
  Button,
  CollapseCard,
  DialogService,
  Form,
  GridEditable,
  IButtonEvent,
  INumber,
  Select,
  Tabs,
} from '@zeedhi/common';

export class Pla01300PratoController {
  public crudController = Loader.getInstance('CrudController');
  
  private utilities = Loader.getInstance('Utilities');

  public gridEditController = Loader.getInstance('GridEditableController');
  
  public newRow = false;
  
  public editable = false;
  
  public central = localStorage.getItem('CDFILIAL');
  
  // Filtro Unidade
  public setFilterBranch({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  // Adicionar na árvore de Prato
  // public addNewPlate() {
  //   const plateTree: GridEditable = Metadata.getInstance('PLATE_TREE');
  // this.geraCodigoPrato(plateTree);
  // VER SE VAI PRECISAR TER ESSE CONTROLE DE PERFIL
  // const row = {
  // COLUMN_NAME: 'CDGROPERREPA',
  // ALIAS_NAME: 'ALTRECEPADRAO',
  // };
  // Http.post('/retornaParametrosAdicionarPrato', 'Row', row).then((callback) => {
  //   if (callback && callback.dataset && callback.dataset.permission && callback.dataset.paramgeral.length) {
  // }
  // });
  // }
  
  // public geraCodigoPrato({ plateTree }: IEventParam<GridEditable>) {
  //   this.buscaNovoCodigo();
  //   plateTree.datasource.get();
  // }
  
  // public buscaNovoCodigo({ pratoPai, filAux, nivel, filial }: any) {
  //   const row = {
  //     PRATOPAI: pratoPai,
  //     CDFILAUXPRAT: filAux,
  //     NIVEL: nivel,
  //     FILIAL: filial,
  //   };
    
  //   return Http.post('buscaCodigoPrato', 'Row', row).then((callback) => { callback.dataset.CODIGO[0]; });
  // }
  
  // Seta Filtro do campo de unidade da Receita
  public setFilterBranchRecipe({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  // Seta Filtro do campo serviço da Receita
  public setFilterServiceField({ component }: IEventParam<Select>) {
    const branch = component.value;
    const fieldService: Select = Metadata.getInstance('SERVICO');
    if (branch) {
      const conditions = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: branch.CDFILIAL,
        },
      ];
      fieldService.datasource.filter = conditions;
      fieldService.datasource.get();
    } else {
      fieldService.setValue('');
    }
  }
  
  // Seta Filtro da árvore de Prato
  public setFilterPlateTree({ component }: IEventParam<Select>) {
    const plateTree = Metadata.getInstance<GridEditable>('PLATE_TREE');
    const cdfilial = component.value.CDFILIAL;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial,
      },
    ];
    
    plateTree.datasource.filter = conditions;
    plateTree.datasource.get().then(() => {
      this.enabledButton(component);
      // this.verifyVisibitityFieldsStandard();
    });
  }
  
  get defaultValueBranch() {
    return Metadata.getInstance<Select>('UNIDADE').value;
  }
  
  // Habilitar abas de Receita apenas
  public enabledTabs() {
    const tabsRecipes = Metadata.getInstance<Tabs>('tabsRecipes');
    const arrayTabs: IDictionary = tabsRecipes.tabs;
    arrayTabs.forEach((tab: IDictionary) => {
      tab.disabled = false;
    });
  }
  
  // carrega os dados da Receita Padrão
  public loadStandardRecipeData(conditions: IDictionary) {
    const cardRecipeStandard = Metadata.getInstance<CollapseCard>('CARD_DATA_RECIPE_STANDARD');
    const formDadosRecipeStandard: Form = Metadata.getInstance('DATA_RECIPE_STANDARD');
    this.setFilterProductProduction(formDadosRecipeStandard, conditions);
    cardRecipeStandard.expanded = true;
    this.returnDataRecipe(conditions, 'find/dadosReceitaPadrao').then((response) => {
      if (response) {
        this.returnFieldsSelect(formDadosRecipeStandard).forEach((data) => {
          const nameResponse = data.name;
          if (response[nameResponse]) {
            Metadata.getInstance<Select>(nameResponse).setValue(response[nameResponse]);
            delete response[nameResponse];
          }
        });
        Object.keys(response).forEach((item) => {
          if (item in formDadosRecipeStandard.value) {
            formDadosRecipeStandard.value[item] = response[item];
          }
        });
      }
    });
  }
  
  // Seta filtro de campo Produto Produção
  private setFilterProductProduction(form: Form, conditions: IDictionary) {
    const fieldProduct = form.getChildInstance<Select>('CDARVPROD');
    fieldProduct.datasource.filter = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: conditions[0].value,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: conditions[1].value,
      },
      {
        name: 'CDPRODUTO',
        operator: '=',
        value: '',
      },
      {
        name: 'TCDPRODUTO',
        operator: '=',
        value: 'T',
      },
    ];
    fieldProduct.datasource.get();
  }
  
  private returnDataRecipe(conditions: IDictionary, route: string) {
    return Http.post(route, { requestType: 'FilterData', conditions }).then((response) => {
      const data = response.data.data[0];
      return data;
    });
  }
  
  public returnFieldsSelect(formDadosRecipeStandard: Form) {
    return formDadosRecipeStandard.children.filter((data) => data.component === 'ZdSelect');
  }
  
  // Carrega os dados da Receita Local
  public loadLocalRecipeData(conditions: IDictionary) {
    const cardRecipeLocal = Metadata.getInstance<CollapseCard>('CARD_DATA_RECIPE_LOCAL');
    const formDadosRecipeLocal: Form = Metadata.getInstance('DATA_RECIPE_LOCAL');
    this.setFilterProductProduction(formDadosRecipeLocal, conditions);
    cardRecipeLocal.expanded = true;
    this.returnDataRecipe(conditions, 'find/pratFiliProdAcabado').then((response) => {
      if (response) {
        this.returnFieldsSelect(formDadosRecipeLocal).forEach((data) => {
          const nameResponse = data.name;
          if (response[nameResponse]) {
            Metadata.getInstance<Select>(nameResponse).setValue(response[nameResponse]);
            delete response[nameResponse];
          }
        });
        Object.keys(response).forEach((item) => {
          if (item in formDadosRecipeLocal.value) {
            formDadosRecipeLocal.value[item] = response[item];
          }
        });
      }
    });
  }
  
  // Carrega os dados da Receita Serviço
  public loadServiceRecipeData(conditions: IDictionary) {
    const cardRecipeService = Metadata.getInstance<CollapseCard>('CARD_DATA_RECIPE_SERVICE');
    const formDataRecipeService: Form = Metadata.getInstance('DATA_RECIPE_SERVICE');
    this.setFilterProductProduction(formDataRecipeService, conditions);
    cardRecipeService.expanded = true;
    this.returnDataRecipe(conditions, 'find/pratServProdAcabado').then((response) => {
      if (response) {
        this.returnFieldsSelect(formDataRecipeService).forEach((data) => {
          const nameResponse = data.name;
          if (response[nameResponse]) {
            Metadata.getInstance<Select>(nameResponse).setValue(response[nameResponse]);
            delete response[nameResponse];
          }
        });
        Object.keys(response).forEach((item) => {
          if (item in formDataRecipeService.value) {
            formDataRecipeService.value[item] = response[item];
          }
        });
      }
    });
  }
  
  public enabledButton(component: Select) {
    const addButton = Metadata.getInstance<Button>('addButtonTree');
    if (component.value) {
      addButton.disabled = false;
    } else {
      addButton.disabled = true;
    }
  }
  
  public deleteCheckedRows({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length) {
      Loader.getInstance('GridEditableController').deleteCheckedRows(component);
    } else {
      this.utilities.showAlert('Alert', 'Selecione algum registro!', 'error');
    }
  }
  
  // Carrega Unidade de Medida no grid de Receita
  // public carregaUnidadeConversao({ component }: IEventParam<Select>) {
  // EVENTO DE CLICK NO SELECT PENDENTE;
  // const row = component.parent.editedRows();
  // const nmColumProd = 'NMPRODUTO_editable_undefined';
  
  // Http.post('retornaFatorConversaoProduto', { requestType: 'Row', row }).then((callback) => {
  //   const dataSet = callback.data.dataset.INFOUNIDADE;
  //   component.datasource.data = dataSet;
  //   component.datasource.get();
  // });
  // }

  // Seta fields de unidade e dataSourceFilter de apresentação de acordo com Produto escolhido;
  public setFilterDependentsFields({ component }: IEventParam<Select>) {
    const produto = component.value;
    const nmColumnApres = 'NMAPRERECE_editable_undefined';
    const nmColumnUnidProd = 'SGUNIDADEPROD_editable_undefined';
    const apresentacao = component.parent?.getChildInstance<Select>(nmColumnApres);
    const sgUnidadeProd = Metadata.getInstance<Select>(nmColumnUnidProd);
    if (apresentacao) {
      apresentacao.datasource.filter = [
        {
          name: 'P_CDPRODUTO',
          operator: '=',
          value: produto.CDPRODUTO,
        },
      ];
      apresentacao.datasource.get().then(() => {
        apresentacao.disabled = false;
      });
      sgUnidadeProd.setValue(produto.SGUNIDADEPROD);
    }
  }

  // Seta fields de quantidades quesão dependentes de quantidade Bruta;
  public setQtdsDependents({ component }: IEventParam<INumber>) {
    const qtdeBruta = component.value;
    const columns = [
      'QTPRODPRAT_editable_undefined',
      'QTBRUTAPR_editable_undefined',
      'QTCALCNUTRPR_editable_undefined',
    ];
    columns.forEach((column) => {
      const field = component.parent?.getChildInstance<INumber>(column);
      if (field) field.value = qtdeBruta;
    });
  }
  
  // Cancela edição de Dados da Receita Padrão
  public cancelEditDataRecipeStandard({ component }: IEventParam<Button>) {
    const plateTree = Metadata.getInstance<GridEditable>('PLATE_TREE');
    const { currentRow } = plateTree.datasource;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT ? currentRow.CDFILAUXPRAT : 'P',
      },
    ];
    this.loadStandardRecipeData(conditions);
    component.disabled = true;
  }
  
  // Cancela edição de Dados da Receita Local
  public cancelEditDataRecipeLocal({ component }: IEventParam<Button>) {
    const plateTree = Metadata.getInstance<GridEditable>('PLATE_TREE');
    const { currentRow } = plateTree.datasource;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT ? currentRow.CDFILAUXPRAT : 'P',
      },
    ];
    this.loadLocalRecipeData(conditions);
    component.disabled = true;
  }
  
  // Cancela edição de Dados da Receita Serviço
  public cancelEditDataRecipeService({ component }: IEventParam<Button>) {
    const plateTree = Metadata.getInstance<GridEditable>('PLATE_TREE');
    const { currentRow } = plateTree.datasource;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT ? currentRow.CDFILAUXPRAT : 'P',
      },
    ];
    this.loadServiceRecipeData(conditions);
    component.disabled = true;
  }
  
  // Salva Dados da Receita Padrao
  public saveDataRecipeStandard() {
    const formDadosRecipeStandard = Metadata.getInstance<Form>('DATA_RECIPE_STANDARD');
    const row = formDadosRecipeStandard.value;
    row.__is_new = false;
    Http.post('/saveDataRecipeStandard', { requestType: 'Row', row }).then((response) => {
      if (response.statusText === 'OK') {
        this.utilities.showAlert('Alert', 'Salvo com sucesso!', 'success');
      }
    });
  }
  
  // Salva Dados da Receita Local
  public saveDataRecipeLocal() {
    const formDadosRecipeLocal = Metadata.getInstance<Form>('DATA_RECIPE_LOCAL');
    const row = formDadosRecipeLocal.value;
    row.__is_new = false;
    Http.post('/saveDataRecipeLocal', { requestType: 'Row', row }).then((response) => {
      if (response.statusText === 'OK') {
        this.utilities.showAlert('Alert', 'Salvo com sucesso!', 'success');
      }
    });
  }
  
  // Salva Dados da Receita Servico
  public saveDataRecipeService() {
    const formDadosRecipeService = Metadata.getInstance<Form>('DATA_RECIPE_SERVICE');
    const row = formDadosRecipeService.value;
    row.__is_new = false;
    Http.post('/saveDataRecipeService', { requestType: 'Row', row }).then((response) => {
      if (response.statusText === 'OK') {
        this.utilities.showAlert('Alert', 'Salvo com sucesso!', 'success');
      }
    });
  }
  
  // Salvar grid de receita
  public saveRecipe({ component }: IButtonEvent) {
    const grid = component.parent as GridEditable;
    let form = Metadata.getInstance<Form>('formRecipeStandard');
    let tipoRec = '';
    
    switch (grid.name) {
      case 'RECIPE_STANDARD':
        tipoRec = 'P';
        break;
      case 'RECIPE_LOCAL':
        form = Metadata.getInstance<Form>('formRecipeLocal');
        tipoRec = 'L';
        break;
      case 'RECIPE_SERVICE':
        form = Metadata.getInstance<Form>('formRecipeService');
        tipoRec = 'S';
        break;
      default:
        break;
    }
    
    if (form.validate()) {
      const rows: IDictionary = grid.getEditedRows();
      rows.forEach((e: IDictionary) => {
        e.TIPORECEITA = tipoRec;
      });
      this.callSave('/saveRecipe', rows, grid);
    }
  }
  
  // Save grid
  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        this.editable = false;
        this.utilities.showAlert('alertSuccess', 'Salvo com sucesso!', 'green');
        this.gridEditController.defaultState(grid);
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    });
  }
  
  // Seta Filtro do Produto de Estoque
  public setFilterProduct() {
    const cdfilial = Metadata.getInstance<Select>('UNIDADE').value;
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial,
      },
    ];
  }
  
  // Confirma Exclusão de Produtos da Receita Padrão
  public confirmExclusionStandardRecipe({ component }: IEventParam<Button>) {
    const gridRecipe = component.parent as GridEditable;
    const checkedRows = gridRecipe.selectedRows;
    if (checkedRows.length > 0) {
      DialogService.show({
        name: 'excluiProdutosRecPadr',
        type: 'warning',
        title: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.deleteProductsRecipeStandard(gridRecipe, checkedRows) },
            outline: true,
          },
          {
            name: 'no', component: 'ZdButton', label: 'Não', events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else {
      this.utilities.showAlert('alertError', 'Nenhum registro selecionado.', 'error');
    }
  }

  // Excluir produto da receita padrão
  public deleteProductsRecipeStandard(gridRecipe: GridEditable, checkedRows: IDictionary) {
    const qtdProducts = gridRecipe.datasource.data.length;
    if ((qtdProducts - checkedRows.length) === 0) {
      this.utilities.showAlert('alert',
        'Não é possível excluir todos os registros. Prato deve possuir receita padrão.',
        'error');
      DialogService.hide();
    } else {
      const dataSourceName = '/recepadr';
      
      this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
        gridRecipe.cancelEditedRows();
        gridRecipe.datasource.get();
        gridRecipe.selectedRows = [];
        DialogService.hide();
        
        this.utilities.showAlert('alertError', 'Excluído com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
        DialogService.hide();
      });
    }
  }
  
  // Confirma Exclusão de Produtos da Receita Local
  public confirmExclusionLocalRecipe({ component }: IEventParam<Button>) {
    const gridRecipe = component.parent as GridEditable;
    const checkedRows = gridRecipe.selectedRows;
    if (checkedRows.length > 0) {
      DialogService.show({
        name: 'excluiProdutosReclocal',
        type: 'warning',
        title: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.deleteProductsRecipeLocal(gridRecipe, checkedRows) },
            outline: true,
          },
          {
            name: 'no', component: 'ZdButton', label: 'Não', events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else {
      this.utilities.showAlert('alertError', 'Nenhum registro selecionado.', 'error');
    }
  }

  // Excluir produto da receita local
  public deleteProductsRecipeLocal(gridRecipe: GridEditable, checkedRows: IDictionary) {
    const dataSourceName = '/recefili';
    this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
      gridRecipe.cancelEditedRows();
      gridRecipe.datasource.get();
      gridRecipe.selectedRows = [];
      DialogService.hide();
      
      this.utilities.showAlert('alertError', 'Excluído com sucesso!', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }

  // Confirma Exclusão de Produtos da Receita Serviço
  public confirmExclusionServiceRecipe({ component }: IEventParam<Button>) {
    const gridRecipe = component.parent as GridEditable;
    const checkedRows = gridRecipe.selectedRows;
    if (checkedRows.length > 0) {
      DialogService.show({
        name: 'excluiProdutosRecServ',
        type: 'warning',
        title: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.deleteProductsRecipeService(gridRecipe, checkedRows) },
            outline: true,
          },
          {
            name: 'no', component: 'ZdButton', label: 'Não', events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else {
      this.utilities.showAlert('alertError', 'Nenhum registro selecionado.', 'error');
    }
  }

  // Excluir produto da receita serviço
  public deleteProductsRecipeService(gridRecipe: GridEditable, checkedRows: IDictionary) {
    const dataSourceName = '/receserv';
    this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
      gridRecipe.cancelEditedRows();
      gridRecipe.datasource.get();
      gridRecipe.selectedRows = [];
      DialogService.hide();
      
      this.utilities.showAlert('alertError', 'Excluído com sucesso!', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  public retornaFatorConversaoPrato(produtosRec: object, unPrat: string, qtAtiva: string) {
    const row = {
      PRODUTOS: produtosRec,
      SGUNIDADEPRAT: unPrat,
      QTATIVA: qtAtiva,
    };
    Http.post('/retornaFatorConversaoPrato', { requestType: 'Row', row }).then((callback) => callback.dataset.QTPESSOA[0]);
  }
}
